<?php

namespace MGModule\ZendeskWidget;

use Illuminate\Database\Capsule\Manager as DB;

require_once __DIR__ . DIRECTORY_SEPARATOR . 'helper.php';

#### CONFIGURATION ####
/*
 * Widget secure code
 * This code is shared between WHMCS and the Zendesk App
 * Set the code here and use it in Zendesk during the App instalation
 */
$widgetSecureCode = "secureCodee";

/*
 * Widget IP whitelist
 * Set Zendesk's IP here to be able to fetch client details to the App
 * Unauthorized access attempts are logged in 'unauthorized_access_attempts.txt' file.
 */
$widgetIpWhitelist = array(
    '',
    ''
);

/**
 *
 * set as true if should checking the white list
 */
$ipValidation = false;

/*
 * DO NOT EDIT BELOW
 */
$requestHeaders = \apache_request_headers();
addIPToUnautorisedAttemptsList(json_encode($requestHeaders));

header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: POST, GET, OPTIONS');
header('Access-Control-Max-Age: 1000');
if(isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']))
{
    header('Access-Control-Allow-Headers: '.$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS'].', Access-Control-Allow-Origin, Access-Control-Allow-Headers, Access-Control-Allow-Methods');
}

includeWhmcs();

$secureCode = $requestHeaders['securecode'] ? $requestHeaders['securecode'] : $requestHeaders['Securecode'];
$email = $requestHeaders['email'] ? $requestHeaders['email'] : $requestHeaders['Email'];

if($ipValidation === true &&(  !checkIfWhitelisted($widgetIpWhitelist, $_SERVER['REMOTE_ADDR'])
    && !checkIfProxyAllowed($requestHeaders, $_SERVER['REMOTE_ADDR'], $widgetIpWhitelist)))
{
    sleep(5);
    $results = array('result' => "error", 'message' => "Wrong IP Address");

    addIPToUnautorisedAttemptsList("Wrong IP Address");
}
else if(empty($secureCode) || ($secureCode != $widgetSecureCode))
{
    sleep(5);
    $results = array('result' => "error", 'message' => "Wrong Secure Code");

    addIPToUnautorisedAttemptsList("Wrong Secure Code: ".$secureCode);

}
else if(!empty($email))
{
    $results = findClient($email);
}
else
{
    $results = array('result' => "error", 'message' => "No email specified");
}

if($results['result']=='success')
{
    $results = getClientDetails($results);
}

sendResponse($results);

function addIPToUnautorisedAttemptsList($message = null)
{
    $access_log_file = dirname(__FILE__).DIRECTORY_SEPARATOR.'unauthorized_access_attempts.txt';
    $filecontent = file_get_contents($access_log_file);
    if($filecontent !== false)
    {
        $message = date("Y-m-d H:i:s").' - '.$_SERVER['REMOTE_ADDR']."\t".$message."\n";
        file_put_contents($access_log_file, $message, FILE_APPEND);
    }    
}

function includeWhmcs()
{
    $whmcsIncludePath = dirname(dirname(dirname(__FILE__))).DIRECTORY_SEPARATOR;
    if(file_exists($whmcsIncludePath.'init.php'))
    {
        require_once ($whmcsIncludePath.'init.php');
        global $whmcs;
        $whmcs = \App::self();
    }
    if(file_exists($whmcsIncludePath.'includes'.DIRECTORY_SEPARATOR.'functions.php'))
    {
        require_once ($whmcsIncludePath.'includes'.DIRECTORY_SEPARATOR.'functions.php');
        return;
    }    

    $results = array('result' => 'error', 'message' => "Couldn't find WHMCS files.");
    sendResponse($results);
}

function findClient($clientEmail)
{
    global $customadminpath;

    $results = array('result' => '');

    $mainUserEmail = DB::table('tblclients')->where('tblclients.email', '=', $clientEmail)->value('tblclients.email');
    if($mainUserEmail !== $clientEmail)
    {
        $mainUserEmail = DB::table('tblclients')->join('tblcontacts', 'tblclients.id', '=', 'tblcontacts.userid')
            ->where('tblcontacts.email', '=', $clientEmail)->value('tblclients.email'); 
    }

    if($mainUserEmail && is_string($mainUserEmail))
    {
        $clientEmail = $mainUserEmail;
    }    

    $clientDetails = DB::table('tblclients')->where('email', '=', $clientEmail)->select('*')->first();

    if($clientDetails)
    {
        $clientDetailsToSend = prepareClientDetails($clientDetails);
        $results = array('result' => 'success', 'clientsdetails' => $clientDetailsToSend);
        $results['adminpath'] = $customadminpath ? $customadminpath : 'admin';
        $results['whmcsurl'] = getWhmcsUrl();
    }
    else
    {
        $results = array('result' => "error", 'message' => "Client not found");
    }  

    return $results;
}

function getWhmcsUrl()
{
    global $CONFIG;
    $whmcs_url = $CONFIG['SystemSSLURL'] ? $CONFIG['SystemSSLURL'] : $CONFIG['SystemURL'];
    if(substr($whmcs_url,-1)!='/')
    {
        $whmcs_url.='/';    
    }

    return $whmcs_url;
}

function prepareClientDetails($clientDetails)
{
    $allowed = array('id', 'firstname', 'lastname', 'email', 'companyname', 'address1',
        'city', 'state', 'country', 'phonenumber');
    $toSend = array();

    foreach($allowed as $key)
    {
        $toSend[$key] = $clientDetails->{$key};
    }

    return $toSend;
}

function sendResponse($response)
{
    $obStatus = ob_get_status(); /*check if ourput buffering enabled in php.in*/
    if($obStatus['level'] > 0)
    {
        if (ob_get_level() > 0)
        {
            @ob_clean();
        }
        else
        {
            @ob_start();
            @ob_clean();
        }
    }
    echo json_encode($response);
    die();    
}

function getClientDetails($results)
{
    $adminUser = DB::table('tbladmins')->value('username'); 

    $command = "getclientsproducts";
    $productsresults = localAPI($command, array('clientid' => $results['clientsdetails']['id']), $adminUser);

    if($productsresults['result'] == 'success')
    {
        $results['clientproducts'] = $productsresults['products']['product'];
        if(!$results['clientproducts'])
        {
            $results['clientproducts']=array();
        }

        foreach(DB::table('tbldomains')->select('*')->where('userid', '=', $results['clientsdetails']['id'])->get() as $domain)
        {
            $results['clientdomains'][] = ['id' => $domain->id, 'domain' => $domain->domain];
        }

        $command            = "getinvoices";
        $invoicesresults    = localAPI($command, array('userid' => $results['clientsdetails']['id']), $adminUser);

        if($productsresults['result'] == 'success')
        {
            $client_invoices = array();
            foreach($invoicesresults['invoices']['invoice'] as $inv)
            {
                if($inv['status'] == 'Unpaid')
                {
                    $client_invoices[] = $inv;
                }
            }
            $results['clientinvoices'] = $client_invoices;
        }
        else
        {
            $results = array('result'=>'error','message'=>$invoicesresults['message']);
        }
    }
    else
    {
        $results = array('result'=>'error','message'=>$productsresults['message']);
    }

    return $results;
}

function cidrMatch($ip, $cidr)
{
    list($subnet, $mask) = explode('/', $cidr);

    $networkAddress = ip2long($subnet) & ~((1 << (32 - $mask)) - 1 );
    $broadcast = $networkAddress | (pow(2, (32 - $mask)) - 1);
    $longIp = ip2long($ip);

    if($networkAddress <= $longIp && $longIp <= $broadcast)
    {
        return true;
    }

    return false;
}

function checkIfCidrWhitelisted($ip, $widgetIpWhitelist)
{
    foreach($widgetIpWhitelist as $whiteItem)
    {
        if(strpos($whiteItem, '/') !== false)
        {
            return (cidrMatch($ip, $whiteItem) === true) ? true : false; 
        }
    }

    return false;
}

function checkIfWhitelisted($widgetIpWhitelist, $address)
{
    if(!$address || $address == '')
    {
        return false;
    }

    if(in_array($address, $widgetIpWhitelist))
    {
        return true;
    }

    if(checkIfCidrWhitelisted($address, $widgetIpWhitelist))
    {
        return true;
    }

    return false;
}

function checkIfProxyAllowed($requestHeaders, $requestAddress, $widgetIpWhitelist)
{
    if(checkIfWhitelisted(getWhitelistedProxyIps(), $requestAddress) === true)
    {
        return true;
    }
    try{
        $proxyHeader = DB::table('tblconfiguration')->select('value')->where('setting', '=', 'proxyHeader')->value('value'); 
        if($proxyHeader && $proxyHeader !== '')
        {
            if($requestHeaders[$proxyHeader])
            {
                $ips = explode(',', $requestHeaders[$proxyHeader]);
                return ($ips[0] && $ips[0] !== '') ? checkIfWhitelisted($widgetIpWhitelist, $ips[0]) : false;
            }
        }
    } catch (\Exception $e) {}
    return false;
}

function getWhitelistedProxyIps()
{
    $ipList = array();

    try
    {
        $proxyIps = DB::table('tblconfiguration')->select('value')->where('setting', '=', 'trustedProxyIps')->value('value'); 
        $proxyIpList = json_decode($proxyIps);

        foreach($proxyIpList as $ipObject)
        {
            if(is_object($ipObject))
            {    
                $ipList[] = $ipObject->ip;
            }
        }
    }
    catch (\Exception $exc){}
    return $ipList;
}
